'// HTML to Delimited file
'// Created 20050224 by Casey Sprangel. Copyright 2005, all rights reserved.
'// Written in PowerBasic CC 4.0 (www.powerbasic.com)
'// This program was designed for Brian Simpson, netmatrix.com
'// --------------------------------------------------------------------------
'// This program will recursively read HTML files in a directory and all sub-
'// directories and output only the BODY content into a single delimited file
'// for importing into a database.
'// --------------------------------------------------------------------------

#COMPILE EXE
#DIM ALL
#INCLUDE "WIN32API.INC"                                     '// include our API calls

%version = 20050225                                         '// This is the version number. Change for every update
$divider = $TAB                                             '// The character used to delim each field in the output file

DECLARE FUNCTION dpush(a AS STRING) AS LONG
DECLARE FUNCTION dpop() AS STRING
DECLARE FUNCTION isdir(filen AS STRING) AS LONG
DECLARE FUNCTION exist(filen AS STRING) AS LONG
DECLARE FUNCTION smartparse(b AS STRING, d AS STRING) AS STRING
DECLARE FUNCTION bodychk(f AS STRING) AS LONG
DECLARE FUNCTION filework(fpath AS STRING) AS LONG

GLOBAL dnames() AS STRING                                   '// holds a list of directory names
GLOBAL dpoint AS LONG                                       '// pointer to the current dname offset
GLOBAL dcount AS LONG                                       '// counts the total number of directories found
GLOBAL ofile AS STRING                                      '// the name of our output file
GLOBAL rpath AS STRING

'//---------------------------------------------------------------------------

FUNCTION dpush(a AS STRING) AS LONG                         '// pushes a directory on to the stack
  DIM i AS LOCAL LONG
  FOR i = 1 TO dpoint                                       '// walk through the known list of directories
    IF UCASE$(dnames(i)) = UCASE$(a) THEN                   '// did we find a dupe?
      EXIT FUNCTION                                         '// leave without adding anything to our list
    END IF
  NEXT
  INCR dpoint                                               '// move the pointer to the next element
  INCR dcount                                               '// total number of directories found
  IF dpoint > UBOUND(dnames()) THEN                         '// is our table to small to hold the new element?
    REDIM PRESERVE dnames(dpoint)                           '// extend the table
  END IF
  dnames(dpoint) = a                                        '// add the new path to the table
  STDOUT "DIR: " + a
END FUNCTION

'//---------------------------------------------------------------------------

FUNCTION dpop() AS STRING                                   '// returns a directory from the stack
  IF dpoint > 0 THEN                                        '// is there anything on the stack?
    FUNCTION = dnames(dpoint)                               '// return the last directory on the stack
    dnames(dpoint) = ""                                     '// blank out the entry - return some memory to working space
    DECR dpoint                                             '// move the pointer down to the previous entry
  ELSE                                                      '// our pointer is at the bottom of the stack
    FUNCTION = ""                                           '// return nothing - stack is empty
  END IF
END FUNCTION

'//---------------------------------------------------------------------------

FUNCTION isdir(filen AS STRING) AS LONG
  '// Returns -1 if a file or path exists and is a directory. Returns 0 otherwise. Does not change DIR$ FindFirst/FindNext pointers
  DIM result AS LOCAL LONG                                  '// a junk variable, we don't care what comes back
  IF RIGHT$(filen, 1) = ":" THEN                            '// does file name end in a colon (i.e. a drive letter)?
    result = GETATTR(filen + "\")                           '// see if the root path exists (adding a slash)
  ELSE                                                      '// this is a normal path or filename
    result = GETATTR(RTRIM$(filen, "\"))                    '// see if it exists, paths with trailing slash marks are trimmed
  END IF
  IF (ERR = 0) THEN                                         '// does it exist?
    IF (result AND 16&) = 16& THEN                          '// is this a directory?
      FUNCTION = -1                                         '// return true, this is a directory
    ELSE
      FUNCTION = 0                                          '// return false, this is not a directory
    END IF
  ELSE                                                      '// file does not exist
    ERRCLEAR                                                '// clear the error flag
    FUNCTION = 0                                            '// return false
  END IF
END FUNCTION

'//---------------------------------------------------------------------------

FUNCTION exist(filen AS STRING) AS LONG
  '// Returns -1 if a file or path exists. Returns 0 if it does not exist. Does not change DIR$ FindFirst/FindNext pointers
  DIM result AS LOCAL LONG                                  '// a junk variable, we don't care what comes back
  IF RIGHT$(filen, 1) = ":" THEN                            '// does file name end in a colon (i.e. a drive letter)?
    result = GETATTR(filen + "\")                           '// see if the root path exists (adding a slash)
  ELSE                                                      '// this is a normal path or filename
    result = GETATTR(RTRIM$(filen, "\"))                    '// see if it exists, paths with trailing slash marks are trimmed
  END IF
  FUNCTION = (ERRCLEAR = 0)                                 '// check to see if we had an error
END FUNCTION

'//---------------------------------------------------------------------------

FUNCTION smartparse(b AS STRING, d AS STRING) AS STRING
  '// Changes delimited string into binary packed string honoring double quoted sections
  LOCAL buf AS STRING                                       '// working string buffer
  LOCAL i AS LONG                                           '// temp pointer
  LOCAL j AS LONG                                           '// temp pointer
  buf = b                                                   '// make a copy because we will modify
  i = TALLY(buf, d)                                         '// count the parts based on passed delim character
  DIM h(i) AS STRING                                        '// make a table large enough to hold our string parts
  j = 0
  DO WHILE LEN(buf) > 0                                     '// process the string into parts
    i = INSTR(buf + d, d)                                   '// locate the next delimiter
    h(j) = TRIM$(LEFT$(buf, i), d)                          '// copy this part into our table
    buf = MID$(buf, i + 1)                                  '// cut off the part we already have
    INCR j                                                  '// move to the next table element
  LOOP
  DECR j                                                    '// change back to zero-based counting
  i = j                                                     '// remember the number of elements in the table
  j = 0                                                     '// start with the first element
  DO WHILE j < i                                            '// loop through all of the found parts
    IF LEFT$(h(j), 1) = $DQ THEN                            '// found an open quote?
      h(j) = MID$(h(j), 2)                                  '// remove the open double quote
      DO WHILE RIGHT$(h(j), 1) <> $DQ AND j + 1 <= i
        h(j) = h(j) + $SPC + h(j + 1)                       '// append the next element to the end of this element
        ARRAY DELETE h(j + 1)                               '// remove the empty element
        DECR i
      LOOP
      IF RIGHT$(h(j), 1) = $DQ THEN                         '// does this string end in a double quote?
        h(j) = LEFT$(h(j), -1)                              '// remove the trailing double quote
      END IF
    END IF
    INCR j                                                  '// look at the next element
  LOOP
  REDIM PRESERVE h(i)                                       '// trim off all of the empty elements we packed
  FUNCTION = JOIN$(h(), BINARY)                             '// pass the string back in packed binary format
END FUNCTION

'//---------------------------------------------------------------------------

FUNCTION bodychk(f AS STRING) AS LONG
  LOCAL hf AS LONG
  LOCAL buf AS STRING
  LOCAL buf2 AS STRING
  LOCAL i AS LONG
  LOCAL j AS LONG
  LOCAL x AS LONG
  LOCAL y AS LONG

  hf = FREEFILE                                             '// get next available file handle
  OPEN f FOR BINARY AS hf                                   '// open the file in raw mode
  j = LOF(hf)                                               '// remember the file size
  IF j > 4194304 THEN                                       '// is file too large?
    i = 4194304                                             '// 4 meg is max size we will read into memory
  ELSE                                                      '// the file is not too large
    i = LOF(hf)                                             '// set read chunk size to file size
  END IF
  GET$ hf, i, buf                                           '// get a chunk of the file
  CLOSE hf                                                  '// release the file handle
  x = INSTR(UCASE$(buf), "<BODY")                           '// locate the open body tag
  y = INSTR(UCASE$(buf), "</BODY")                          '// locate the close body tag
  IF x > 0 THEN                                             '// is there an open body tag?
    IF y = 0 THEN                                           '// there is no close body tag?
      IF i <> j THEN                                        '// we did not read the whole file?
        STDOUT "ERROR: Source HTML File (" + f + ") is too large!"
        STDOUT "       This file has been ignored."
        EXIT FUNCTION
      END IF
      STDOUT "Warning: Source HTML file (" + f + ") has open BODY, but no close."
      STDOUT "         Adding close body tag at end of file."
    ELSE
      buf = LEFT$(buf, y - 1)                               '// cut off the close body tag - will add it back later
    END IF
    buf = MID$(buf, x)                                      '// forget everything before the open body tag
    REPLACE $TAB WITH $SPC IN buf                           '// all tabs become spaces
    REPLACE $CRLF WITH $SPC IN buf                          '// all line feeds become a single space
    REPLACE $CR WITH $SPC IN buf                            '// any remaining carriage returns become a single space
    REPLACE $LF WITH $SPC IN buf                            '// any remaining line feeds become a single space
    buf2 = MID$(f, LEN(rpath))                              '// get the relative path and filename
    x = INSTR(-1, buf2, ".")                                '// locate last period in filename
    IF x > 0 THEN                                           '// did we find a period?
      buf2 = LEFT$(buf2, x - 1)                             '// cut off extension
    END IF
    REPLACE "\" WITH "/" IN buf2                            '// flip the slash marks to unix
    hf = FREEFILE                                           '// get next available file handle (may change from above)
    OPEN ofile FOR APPEND AS hf                             '// append our new record to the output file
    PRINT #hf, buf2 + $TAB + buf + "</body>"                '// dump info to file
    CLOSE hf                                                '// release the file handle
  END IF
END FUNCTION

'//---------------------------------------------------------------------------

FUNCTION filework(fpath AS STRING) AS LONG
  LOCAL buf AS STRING                                       '// working string buffer
  LOCAL rpath AS STRING                                     '// relative path to starting directory
  DIM lpFindFileData AS WIN32_FIND_DATA                     '// structure for getting file info
  LOCAL path AS STRING                                      '// file path location
  LOCAL filecount AS LONG                                   '// total file names found
  LOCAL hd1 AS LONG                                         '// this is the directory handle for finding file names and attributes
  DIM f() AS LOCAL STRING                                   '// holds all of the files we find
  LOCAL fp AS LONG                                          '// pointer to the total number of file names we have in our table
  LOCAL fwork AS LONG                                       '// for walking through the files we find

  rpath = RTRIM$(fpath, "\")                                '// remember starting directory - this becomes the relative root
  buf = RTRIM$(fpath, "\") + "\*.*"                         '// make a copy because we will modify
  IF INSTR(buf,"\") > 0 THEN                                '// is there a path?
    path = buf                                              '// copy user input
    path = LEFT$(path, INSTR(-1, path, "\"))                '// cut off everything after the last slash
    buf = MID$(buf, LEN(path) + 1)                          '// cut path off of filename
  ELSE                                                      '// no path given?
    path = RTRIM$(CURDIR$, "\") + "\"                       '// use our local directory instead
  END IF
  STDOUT ">>" + path + buf                                  '// show where we are working
  IF buf <> "*.*" THEN                                      '// we are asked to look for directories but don't have a wildcard that would work for directories too?
    hd1 = FindFirstFile( path + "*.*", lpFindFileData)      '// find first file in this directory
    IF hd1 <> %INVALID_HANDLE_VALUE THEN                    '// did we get a valid directory handle?
      DO                                                    '// start working our loop
        IF (lpFindFileData.dwFileAttributes AND %FILE_ATTRIBUTE_DIRECTORY) = %FILE_ATTRIBUTE_DIRECTORY THEN '// is this a directory?
          IF lpFindFileData.cFileName = "." OR lpFindFileData.cFileName = ".." THEN '// did we find the placeholder entries?
            IF FindNextFile(hd1, lpFindFileData) = 0 THEN   '// did we fail to find another file in this directory?
              EXIT LOOP                                     '// leave, no more files to find in this directory
            END IF
            ITERATE                                         '// go work on next file
          END IF
          buf = path + lpFindFileData.cFileName             '// move the file name into our string buffer
          dpush buf                                         '// push our new directory name on to the stack
        END IF
        IF FindNextFile(hd1, lpFindFileData) = 0 THEN       '// no more files in this directory?
          EXIT LOOP                                         '// leave the loop
        END IF
      LOOP
      FindClose hd1                                         '// we are done with this directory, close the handle
    END IF                                                  '// did we get a valid directory handle
  END IF
  hd1 = FindFirstFile( path + buf, lpFindFileData)          '// find first file in this directory using user file name mask
  IF hd1 <> %INVALID_HANDLE_VALUE THEN                      '// did we find a match?
    DO                                                      '// start working loop
      IF (lpFindFileData.dwFileAttributes AND %FILE_ATTRIBUTE_DIRECTORY) = %FILE_ATTRIBUTE_DIRECTORY THEN '// is this a directory?
        IF lpFindFileData.cFileName = "." OR lpFindFileData.cFileName = ".." THEN '// did we find the placeholder entries?
          IF FindNextFile(hd1, lpFindFileData) = 0 THEN     '// did we fail to find another file in this directory?
            EXIT LOOP                                       '// leave, no more files to find in this directory
          END IF
          ITERATE                                           '// go work on next file
        END IF
        buf = path + lpFindFileData.cFileName               '// move the file name into our string buffer
        dpush buf                                           '// push our new directory name on to the stack
      ELSE                                                  '// this is not a directory
        IF LEN(lpFindFileData.cFileName) > 0 THEN           '// is the name NOT empty?
          buf = lpFindFileData.cFileName                    '// get the full file name
          IF INSTR(buf, "\") > 0 THEN                       '// is there a path part of this file name?
            buf = RIGHT$(buf, INSTR(-1, buf, "\") + 1)      '// cut off path info
          END IF
          INCR fp                                           '// add one to the pointer
          IF fp > UBOUND(f()) THEN                          '// is the pointer more than the table size?
            REDIM PRESERVE f(1000 + fp)                     '// add more elements to our table
          END IF
          f(fp) = buf                                       '// put the name on the stack
        END IF                                              '// name not empty?
      END IF
      IF FindNextFile(hd1, lpFindFileData) = 0 THEN         '// no more files in this directory?
        EXIT LOOP                                           '// leave the loop
      END IF
    LOOP
    FindClose hd1                                           '// we are done with this directory, close the handle
  END IF                                                    '// did we get a valid directory handle
  REDIM PRESERVE f(fp)                                      '// cut the table down to just the elements we found (keep memory use small)
  STDOUT "Found " + FORMAT$(fp) + " files in " + fpath
  FOR fwork = 1 TO fp                                       '// walk through all of the files we found
    buf = f(fwork)                                          '// get the file name to work on
    INCR filecount                                          '// count the number of files we process
    bodychk path + buf
  NEXT
END FUNCTION

'//---------------------------------------------------------------------------

FUNCTION PBMAIN () AS LONG
  DIM buf AS STRING
  DIM i AS LONG
  DIM j AS LONG
  DIM fpath AS STRING

  STDOUT "HTML to Delimiter Seperated File version " + FORMAT$(%version)
  buf = COMMAND$                                            '// get command line
  IF INSTR(buf, "/?") > 0 OR buf = "" THEN
    STDOUT ""
    STDOUT "This program will recursively read HTML files in a directory and all sub-"
    STDOUT "directories and output only the BODY content into a single delimited file"
    STDOUT "for importing into a database."
    STDOUT ""
    STDOUT "Syntax:  HTML2DSV <path> <filename.ext>"
    STDOUT ""
    STDOUT "  path        = This is the path to start looking for HTML files."
    STDOUT "                Every file will be checked no matter what extension"
    STDOUT "                will be used. All files with an open BODY tag will"
    STDOUT "                be imported. If this path contains spaces, it must"
    STDOUT "                be enclosed in double quotes."
    STDOUT ""
    STDOUT " filename.ext = This is the output file that will contain all of the"
    STDOUT "                content from the body tags. Read the documentation"
    STDOUT "                for details about the format of this file. If this"
    STDOUT "                filename contains spaces, it must be enclosed in"
    STDOUT "                double quotes."
    EXIT FUNCTION
  END IF
  buf = smartparse(buf, $SPC)                               '// change to packed binary format honoring quoted sections
  i = PARSECOUNT(buf, BINARY)                               '// count the number of parameters
  IF i <> 2 THEN                                            '// not the right number of parameters?
    STDOUT "Error: Invalid number of command line parameters. Use /? for help."
    STDOUT  "Found " + FORMAT$(i) + " parameters"
    FOR j = 1 TO i
      STDOUT "Part " + FORMAT$(j) + ": " + PARSE$(buf, BINARY, j)
    NEXT
    EXIT FUNCTION
  END IF
  fpath = PARSE$(buf, BINARY, 1)                            '// get path
  ofile = PARSE$(buf, BINARY, 2)                            '// get output filename
  STDOUT "Searching: " + fpath                              '// tell user what we are doing
  STDOUT "Writing: " + ofile
  IF exist(ofile) THEN                                      '// is output file here?
    KILL ofile                                              '// erase it so we can start fresh with a new one
  END IF
  buf = fpath                                               '// start with this directory
  rpath = RTRIM$(fpath, "\") + "\"                          '// make sure this ends in a back slash
  DO WHILE LEN(buf) > 0                                     '// keep working through all of the directories
    filework buf                                            '// do the file work
    buf = dpop                                              '// get next directory
  LOOP
END FUNCTION
